/*
 * TM1637.c
 *
 *  Created on: Oct 25, 2023
 *      Author: martin
 */
/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "COPFree.h"
#include "PTB.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "TM1637.h"
#include "Delays.h"

const char segmentMap0[] = {0x00};

const char segmentMap[] = {
    0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07, // 0-7
    0x7f, 0x6f, 0x77, 0x7c, 0x39, 0x5e, 0x79, 0x71, // 8-9, A-F
    0x00
};

void TM1637_Init(void)
{
    TM1637_SetBrightness(7);
    TM1637_SetBrightness(8);
}

void TM1637_DisplayDecimal(int number, short int displaySeparator, short int show0)
{
    unsigned char digitArr[4];
     int v=number;
    uint8_t i = 0;
    for ( i = 0; i < 4; ++i) {
    	
            if((show0==1) && (v==0)) digitArr[i] = segmentMap0[v % 10];		
            else digitArr[i] = segmentMap[v % 10];

           if (i == 2 && displaySeparator) {
             digitArr[i] |= 1 << 7;
          }
        v /= 10;
    }

    TM1637_Start();
    TM1637_WriteByte(0x40);
    TM1637_ReadResult();
    TM1637_Stop();

    TM1637_Start();
    TM1637_WriteByte(0xc0);
    TM1637_ReadResult();

    for (i = 0; i < 4; ++i) {
              if(number==0) digitArr[0]=0x3f;
              TM1637_WriteByte(digitArr[3 - i]);
        TM1637_ReadResult();
    }

    TM1637_Stop();
}


// Valid brightness values: 0 - 8.
// 0 = display off.
void TM1637_SetBrightness(char brightness)
{
    // Brightness command:
    // 1000 0XXX = display off
    // 1000 1BBB = display on, brightness 0-7
    // X = don't care
    // B = brightness
    TM1637_Start();
    TM1637_WriteByte(0x87 + brightness);
    TM1637_ReadResult();
    TM1637_Stop();
}

void TM1637_Start(void)
{
    PORTB1_bit=1; //CLK_HIGH
    PORTB0_bit=1;//DATA HIGH
    asm nop;
    asm nop;
    asm nop; // 1.953us 
    asm nop;
    asm nop;
    asm nop;
    asm nop;
    PORTB0_bit=0; //DATA_LOW
}

void TM1637_Stop(void)
{
    PORTB1_bit=0; //CLK LOW
    asm nop;
    asm nop;
    asm nop; // 1.953us 
    asm nop;
    asm nop;
    asm nop;
    asm nop;
    PORTB0_bit=0; //DATA_LOW
    asm nop;
    asm nop;
    asm nop; // 1.953us 
    asm nop;
    asm nop;
    asm nop;
    asm nop;
    PORTB1_bit=1; //CLK HIGH
    asm nop;
    asm nop;
    asm nop; // 1.953us 
    asm nop;
    asm nop;
    asm nop;
    asm nop;  
    PORTB0_bit=1; //DATA HIGH
}

void TM1637_ReadResult(void)
{
    PORTB1_bit=0; //CLK LOW
    Delay_5us();
    PORTB1_bit=1; //CLK HIGH
    asm nop;
    asm nop;
    asm nop; // 1.953us 
    asm nop;
    asm nop;
    asm nop;
    asm nop;
    PORTB1_bit=0; //CLK LOW
}

void TM1637_WriteByte(unsigned char b)
{       
  short int i = 0;
        for (i = 0; i < 8; ++i) {
                 PORTB1_bit=0; //CLK LOW
                if (b & 0x01) {
                        PORTB0_bit=1; //DATA HIGH
                }
                else {
                        PORTB0_bit=0; //DATA_LOW
                }
                Delay_3us();
                b >>= 1;
                 PORTB1_bit=1; //CLK HIGH
                 Delay_3us();
        }
}
